/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#include "CoreMinimal.h"
#include "Runtime/Launch/Resources/Version.h"
#if ENGINE_MAJOR_VERSION == 4 && ENGINE_MINOR_VERSION >= 18
#include "UObject/SoftObjectPath.h"
#else
#include "Misc/StringClassReference.h"
#endif

#include "PluginProjectSettings.generated.h"

#if ENGINE_MAJOR_VERSION == 4 && ENGINE_MINOR_VERSION >= 25
#define UPROPCLASS FProperty
#else
#define UPROPCLASS UProperty
#endif

UENUM()
namespace EMixCastClipMode
{
	enum Type
	{
		PerPixel,
		Planar
	};
}

UENUM()
namespace EMixCastTranslucencyMode
{
	enum Type
	{
		None,
		ForceAdditive,
		FromAlpha
	};
}

UCLASS(config = Game, defaultconfig)
class MIXCAST_API UPluginProjectSettings : public UObject
{
	GENERATED_BODY()
public:
	UPluginProjectSettings(const FObjectInitializer& ObjectInitializer);

	UPROPERTY(config, noclear, EditDefaultsOnly, BlueprintReadWrite, Category = "Customization", meta = (MetaClass = "ExpCamera"))
		FStringClassReference CameraClass;
	UPROPERTY(config, noclear, EditDefaultsOnly, BlueprintReadWrite, Category = "Customization", meta = (MetaClass = "ExpVideoInput"))
		FStringClassReference VideoInputClass;
	UPROPERTY(config, noclear, EditDefaultsOnly, BlueprintReadWrite, Category = "Customization", meta = (MetaClass = "ExpViewfinder"))
		FStringClassReference ViewfinderClass;

	UPROPERTY(EditAnywhere, config, Category = "Rendering")
		TEnumAsByte<EMixCastClipMode::Type> ClipMode = EMixCastClipMode::PerPixel;

	UPROPERTY(EditAnywhere, config, Category = "Rendering")
		TEnumAsByte<EMixCastTranslucencyMode::Type> TranslucencyMode = EMixCastTranslucencyMode::None;

	UPROPERTY(EditAnywhere, config, Category = "Rendering")
		bool CanRenderOpaqueBg = true;
	UPROPERTY(EditAnywhere, config, Category = "Rendering")
		bool CanRenderTransparentBg = false;

	//UPROPERTY(EditAnywhere, config, Category = "Rendering", meta = (HideAlphaChannel))
		FLinearColor BackgroundColor = FLinearColor::Black;

	UPROPERTY(EditAnywhere, config, Category = "Build")
		bool bRequireCommandLineFlag;

	void ValidateCurrentSettings();

#if UE_EDITOR
	static void Register();
	static void Unregister();

	virtual bool CanEditChange(const UPROPCLASS* InProperty) const override;
	virtual void PostEditChangeProperty(struct FPropertyChangedEvent& PropertyChangedEvent) override;

	void EnforceGlobalClipPlaneSettingIfNeeded();
#endif
};
