/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#include "MixCast.h"
#include "MixCastLog.h"
#include "Data/SdkData.h"
#include "Plugin/PluginAnalytics.h"
#include "Plugin/PluginUpdater.h"

#include "Runtime/Launch/Resources/Version.h"
#include "HAL/IConsoleManager.h"
#if ENGINE_MAJOR_VERSION == 4 && ENGINE_MINOR_VERSION >= 20
#include "Core/Public/Misc/CoreDelegates.h"
#else
#include "CoreDelegates.h"
#endif

IMPLEMENT_MODULE(FMixCast, MixCast)

FMixCast::FMixCast()
{
	bActive = false;
}

void FMixCast::StartupModule()
{
#if UE_EDITOR
	UPluginProjectSettings::Register();
	FPluginAnalytics::SendReport();
	FPluginUpdater::CheckForUpdates(true);
#endif

	if (FModuleManager::Get().IsModuleLoaded("SteamVR"))
		FModuleManager::Get().LoadModule(FName("MixCast_SteamVR"));
	if (FModuleManager::Get().IsModuleLoaded("OculusHMD"))
		FModuleManager::Get().LoadModule(FName("MixCast_Oculus"));

	mixcast::thrift::ThriftUtil::SetUpLogCallback();

	ConfigReadyDelegate = FCoreDelegates::ConfigReadyForUse.AddUObject(SdkData::GetProjectSettings(), &UPluginProjectSettings::ValidateCurrentSettings);
	if (GConfig && GConfig->IsReadyForUse())
		SdkData::GetProjectSettings()->ValidateCurrentSettings();
}

void FMixCast::ShutdownModule()
{
	if (IsActive())
		SetActive(false);

	FCoreDelegates::ConfigReadyForUse.Remove(ConfigReadyDelegate);
	ConfigReadyDelegate.Reset();

#if UE_EDITOR
	UPluginProjectSettings::Unregister();
#endif
}

void FMixCast::SetActive(bool active)
{
	if (bActive == active)
		return;

	bActive = active;
	if (bActive)
		SdkData::GetProjectSettings()->ValidateCurrentSettings();

	ActiveChanged.Broadcast();
}

bool FMixCast::SendExperienceEvent(const FString& EventId)
{
	if (bActive)
	{
		return clientConnection->SendExperienceEvent(EventId);
	}
	return false;
}

void FMixCast::RegisterSdkActor(AMixCastSdkActor* actor)
{
	if (activeSdkActors.Contains(actor)) {
		MIXCAST_LOG("Actor already registered!");
		return;
	}

	if (activeSdkActors.Num() == 0)
	{
		SdkData::GetProjectSettings()->ValidateCurrentSettings();
		clientConnection = new FServiceCommunicator();
	}

	activeSdkActors.Add(actor);

	activeWorld = activeSdkActors[activeSdkActors.Num() - 1]->GetWorld();
}
void FMixCast::UnregisterSdkActor(AMixCastSdkActor* actor)
{
	if (!activeSdkActors.Contains(actor)) {
		MIXCAST_LOG("Actor not registered!");
		return;
	}

	activeSdkActors.Remove(actor);

	if (activeSdkActors.Num() == 0) {
		SetActive(false);
		activeWorld = nullptr;
		delete clientConnection;
		clientConnection = nullptr;
	}
	else {
		activeWorld = activeSdkActors[activeSdkActors.Num()-1]->GetWorld();
	}
}
