/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#include "ForegroundCutoffMeshComponent.h"
#include "MixCastFrameCanvas.h"
#include "Transmission/Generated/VirtualCamera_types.h"
#include "FrameTimer.hpp"

#include "CoreMinimal.h"
#include "GameFramework/Actor.h"
#include "Components/SceneComponent.h"
#include "Components/StaticMeshComponent.h"
#include "Components/DrawFrustumComponent.h"
#include "Engine/TextureRenderTarget2D.h"
#include "Engine/SceneCapture2D.h"
#include "TimerManager.h"

#include <string>

#include "ExpCamera.generated.h"

using namespace mixcast::data;
using namespace mixcast::thrift;

class FMixCast;
class FExpFrameSender;

UCLASS(HideCategories = ("Actor", "Tick", "HLOD", "Input", "Replication"))
class MIXCAST_API AExpCamera : public AActor
{
	GENERATED_UCLASS_BODY()

public:
	std::string Identifier;

	DECLARE_EVENT_TwoParams(AExpCamera, FFrameStartedEvent, AExpCamera*, VirtualCamera*)
	static FFrameStartedEvent& OnFrameStarted() { return FrameStarted; }
	DECLARE_EVENT_OneParam(AExpCamera, FFrameEndedEvent, AExpCamera*)
	static FFrameEndedEvent& OnFrameEnded() { return FrameEnded; }

	void SetActorHidden(AActor* actor);

	void CaptureIfNeeded();

	float GetNearClipPlane();
	FFrameTimer* GetTimer() { return timer; }

	UMixCastFrameCanvas* GetLayersCanvas() { return LayerCanvas; }

protected:
	static FFrameStartedEvent FrameStarted;
	static FFrameEndedEvent FrameEnded;

	FMixCast* sdkModule;

	UPROPERTY()
		USceneComponent* CameraTransform;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Full Capture")
		USceneCaptureComponent2D* FullCapture;
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Full Capture")
		USceneCaptureComponent2D* FullAlphaCapture;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Foreground Capture")
		USceneCaptureComponent2D* ForegroundCapture;
	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Foreground Capture")
		USceneCaptureComponent2D* ForegroundAlphaCapture;
	UPROPERTY(VisibleAnywhere, BlueprintReadWrite, Category = "Foreground Capture")
		UForegroundCutoffMeshComponent* ForegroundCutoffMesh;

	UPROPERTY(VisibleAnywhere, BlueprintReadOnly, Category = "Textures")
		UMixCastFrameCanvas* LayerCanvas;

	void BeginPlay() override;
	void EndPlay(EEndPlayReason::Type EndPlayReason) override;

	void CaptureFrame(VirtualCamera* camData);

	void UpdateTransform(VirtualCamera* camInfo);
	void UpdateCameraParameters(VirtualCamera* camInfo);
	void ResetHiddenActors();

	void TriggerSceneCaptures(VirtualCamera* camInfo);
	void TriggerFrameComposite();

	float CalculateForegroundDistance(VirtualCamera* camInfo);

private:
	FFrameTimer* timer;
	FExpFrameSender* framePipe;

	bool fullRenderSetAsTransparent;

	USceneCaptureComponent2D* CreateMainCaptureComponent(bool isForeground);
	USceneCaptureComponent2D* CreateAlphaCaptureComponent(bool isForeground);

	void CreateLayerCanvas(VirtualCamera* camInfo);
	void DestroyLayerCanvas();

	void SetCaptureTransparency(USceneCaptureComponent2D* capture, bool backgroundTransparent);
};
