/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#include "Transmission/SharedTextures/SharedTextureReceiver.h"

#include "CoreMinimal.h"
#include "GameFramework/Actor.h"
#include "Components/SceneComponent.h"
#include "ProceduralMeshComponent.h"
#include "Materials/MaterialInstanceDynamic.h"

#include <string>

#include "ExpVideoInput.generated.h"

class AExpCamera;
namespace mixcast {
	namespace data {
		class VirtualCamera;
	}
}

using namespace mixcast::data;

UCLASS(HideCategories = ("Actor", "HLOD", "Input"))
class MIXCAST_API AExpVideoInput : public AActor
{
	GENERATED_UCLASS_BODY()

public:
	std::string Identifier;

protected:
	SharedTextureReceiver* ColorReceiver;
	SharedTextureReceiver* DepthReceiver;

	UPROPERTY()
		USceneComponent* VideoInputTransform;
	UPROPERTY(VisibleAnywhere, Category = "Display")
		UProceduralMeshComponent* ProjectionMesh;
	UPROPERTY(EditDefaultsOnly, BlueprintReadWrite, Category = "Display")
		UMaterialInterface* ProjectionMatTemplate;
	UPROPERTY(VisibleInstanceOnly, BlueprintReadWrite, Category = "Display")
		UMaterialInstanceDynamic* ProjectionMatInstance;

	UFUNCTION(BlueprintCallable, Category = "MixCast")
		UTexture2D* GetColorTexture();
	UFUNCTION(BlueprintCallable, Category = "MixCast")
		UTexture2D* GetDepthTexture();

	void BeginPlay() override;
	void EndPlay(EEndPlayReason::Type EndPlayReason) override;
	void Tick(float DeltaSeconds) override;

	void HandleFrameStarted(AExpCamera* cam, VirtualCamera* camInfo);

	void HandleColorTextureChanged();
	void HandleDepthTextureChanged();

	void RegenerateMesh(int cellsX, int cellsY);

private:
	int lastMeshWidth;
	int lastMeshHeight;

	FDelegateHandle FrameStartedDelegate;
};
