/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#include "CoreMinimal.h"
#include "Engine/Engine.h"

#if PLATFORM_WINDOWS
#include "Windows/AllowWindowsPlatformTypes.h"
#endif
#include <d3d11.h>
#if PLATFORM_WINDOWS
#include "Windows/HideWindowsPlatformTypes.h"
#endif

#include "Transmission/Generated/SharedTex_types.h"

#include <string>

using namespace mixcast::thrift;

/**
 * Handles the creation and management of a texture shared from another program
 */
class SharedTextureReceiver
{
public:
	SharedTextureReceiver(FString texId);
	~SharedTextureReceiver();

	void EnqueueUpdate();
	UTexture2D* GetTexture() { return Texture; }

	DECLARE_EVENT(SharedTextureReceiver, FTextureReferenceChangedEvent)
	FTextureReferenceChangedEvent& OnTextureReferenceChanged() {
		return TextureReferenceChanged;
	}

private:
	FString TextureId;
	SharedTex TextureInfo;
	UTexture2D* Texture;

	ID3D11Resource* SharedResource;

	FDelegateHandle EventHandle;

	void RefreshTextureObject();
	void RefreshTextureFromEvent(std::string TexId, SharedTex TexInfo);
	void DestroyTexture();

	FTextureReferenceChangedEvent TextureReferenceChanged;
};
