/**********************************************************************************
* Blueprint Reality Inc. CONFIDENTIAL
* 2020 Blueprint Reality Inc.
* All Rights Reserved.
*
* NOTICE:  All information contained herein is, and remains, the property of
* Blueprint Reality Inc. and its suppliers, if any.  The intellectual and
* technical concepts contained herein are proprietary to Blueprint Reality Inc.
* and its suppliers and may be covered by Patents, pending patents, and are
* protected by trade secret or copyright law.
*
* Dissemination of this information or reproduction of this material is strictly
* forbidden unless prior written permission is obtained from Blueprint Reality Inc.
***********************************************************************************/

#pragma once

#include "CoreMinimal.h"
#include "Modules/ModuleInterface.h"
#include "Modules/ModuleManager.h"
#include "openvr.h"

class FMixCast_SteamVR : public IModuleInterface
{
public:
	static inline FMixCast_SteamVR& Get()
	{
		return FModuleManager::LoadModuleChecked< FMixCast_SteamVR >( "MixCast_SteamVR" );
	}

	static inline bool IsAvailable()
	{
		return FModuleManager::Get().IsModuleLoaded( "MixCast_SteamVR" );
	}

	virtual void StartupModule() override;
	virtual void ShutdownModule() override;

	void HandleMixCastActiveChanged();
	void HandleTrackingSpaceChanged();
	void HandleMixCastAboutToRender();

private:
	FDelegateHandle mixcastActiveDelegate;
	FDelegateHandle trackingSpaceChangedDelegate;
	FDelegateHandle beforeFrameDelegate;

	bool firstTickComplete = false;
	FTransform trackingToPlayArea;

	bool CalculateHmdToWorldTransform(FTransform& trans);
	bool CalculateTrackingOriginToHmdTransform(FTransform& trans);
	bool CalculateTrackingOriginToStandingTransform(FTransform& trans);
	bool CalculateStandingToPlayAreaTransform(FTransform& trans);

	//Conversion funcs
	static FMatrix ToFMatrix(const vr::HmdMatrix34_t& tm) 
	{
		// Rows and columns are swapped between vr::HmdMatrix34_t and FMatrix
		return FMatrix(
			FPlane(tm.m[0][0], tm.m[1][0], tm.m[2][0], 0.0f),
			FPlane(tm.m[0][1], tm.m[1][1], tm.m[2][1], 0.0f),
			FPlane(tm.m[0][2], tm.m[1][2], tm.m[2][2], 0.0f),
			FPlane(tm.m[0][3], tm.m[1][3], tm.m[2][3], 1.0f));
	}

	//Taken from SteamVRHMD.cpp
	/** Coverts a SteamVR-space vector to an Unreal-space vector.  Does not handle scaling, only axes conversion */
	FORCEINLINE static FVector CONVERT_STEAMVECTOR_TO_FVECTOR(const vr::HmdVector3_t InVector)
	{
		return FVector(-InVector.v[2], InVector.v[0], InVector.v[1]);
	}
};

